==> local-booter-2015.2.6-sources.jar: com/xebialabs/deployit/booter/local/TypeSystemBootstrapper.java <==
package com.xebialabs.deployit.booter.local;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;
import com.google.common.base.Predicate;
import com.google.common.collect.Iterators;

import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.reflect.DescriptorRegistry;
import com.xebialabs.deployit.plugin.api.udm.ConfigurationItem;

import nl.javadude.scannit.Scannit;

import static com.google.common.base.Predicates.equalTo;
import static com.google.common.collect.FluentIterable.from;
import static com.google.common.collect.Lists.newArrayList;
import static com.xebialabs.deployit.booter.local.SyntheticHelper.readSyntheticDocument;
import static com.xebialabs.deployit.booter.local.utils.XmlUtils.childByName;
import static com.xebialabs.xlplatform.utils.ClassLoaderUtils$.MODULE$;

class TypeSystemBootstrapper {

    static void bootstrap() {
        logger.info("Bootstrapping the XL Deploy Type System.");
        TypeDefinitions typeDefs = new TypeDefinitions();
        scanClasses(typeDefs);
        scanSynthetics(typeDefs);
        createTypeTree(typeDefs);
        registerTypeDefinitions(typeDefs);
    }

    private static void createTypeTree(TypeDefinitions typeDefs) {
        for (TypeDefinitions.TypeDefinition typeDefinition : typeDefs.getDefinitions()) {
            typeDefinition.registerTypeTree();
        }
    }

    static void verifyTypes(Verifications verifications) {
        for (Descriptor descriptor : from(DescriptorRegistry.getDescriptors()).filter(new Predicate<Descriptor>() {
            public boolean apply(Descriptor input) {
                return !input.isVirtual();
            }
        })) {
            ((LocalDescriptor) descriptor).verify(verifications);
        }
    }

    private static void scanClasses(TypeDefinitions typeDefs) {
        typeDefs.defineType(ConfigurationItem.class);
        Set<Class<? extends ConfigurationItem>> cis = Scannit.getInstance().getSubTypesOf(ConfigurationItem.class);
        for (Class<? extends ConfigurationItem> ci : cis) {
            typeDefs.defineType(ci);
        }
    }

    private static void scanSynthetics(TypeDefinitions typeDefs) {
        try {
            List<Element> types = newArrayList();
            List<Element> typeModifications = newArrayList();
            readSynthetics("synthetic.xml", types, typeModifications);
            readSynthetics("synthetic-test.xml", types, typeModifications);

            parseAllTypeDefinitions(types, typeDefs);
            parseAllTypeModifications(typeDefs, typeModifications);
        } catch (IOException ex) {
            throw new RuntimeException("Could not read synthetic type definitions.", ex);
        }
    }

    private static void registerTypeDefinitions(TypeDefinitions typeDefs) {
        for (TypeDefinitions.TypeDefinition typeDefinition : typeDefs.getDefinitions()) {
            typeDefinition.register(typeDefs);
        }
    }

    private static void parseAllTypeDefinitions(List<Element> types, TypeDefinitions typeDefs) {
        for (Element type : types) {
            typeDefs.defineType(type);
            Iterator<Element> generatedTypes = childByName(type, equalTo("generate-deployable"));
            if (generatedTypes.hasNext()) {
                typeDefs.defineGeneratedType(generatedTypes.next(), type);
            }

            findAllGeneratedParameterTypes(typeDefs, type);
        }
    }

    private static void findAllGeneratedParameterTypes(final TypeDefinitions typeDefs, final Element type) {
        Iterator<Element> methodDefs = childByName(type, equalTo("method"));
        while (methodDefs.hasNext()) {
            Element methodDef = methodDefs.next();
            Iterator<Element> parameters = childByName(methodDef, equalTo("parameters"));
            if (parameters.hasNext()) {
                typeDefs.defineGeneratedParameterType(methodDef, type);
            }
        }
    }

    private static void parseAllTypeModifications(TypeDefinitions typeDefs, List<Element> typeModifications) {
        for (Element typeModification : typeModifications) {
            typeDefs.modifyType(typeModification);
            findAllGeneratedParameterTypes(typeDefs, typeModification);
        }
    }

    private static void readSynthetics(String name, List<Element> typeDefinitionCollector, List<Element> typeModificationsCollector) throws IOException {
        Enumeration<URL> syntheticXMLs = MODULE$.classLoader().getResources(name);
        while (syntheticXMLs.hasMoreElements()) {
            final URL syntheticXML = syntheticXMLs.nextElement();
            logger.debug("Scanning synthetic XML: {}", syntheticXML);
            Element docElement = readSyntheticDocument(syntheticXML).getDocumentElement();

            Iterators.addAll(typeDefinitionCollector, childByName(docElement, equalTo("type")));
            Iterators.addAll(typeModificationsCollector, childByName(docElement, equalTo("type-modification")));
        }
    }

    private static final Logger logger = LoggerFactory.getLogger(TypeSystemBootstrapper.class);
}

==> local-booter-2015.7.0-sources.jar: com/xebialabs/deployit/booter/local/TypeSystemBootstrapper.java <==
package com.xebialabs.deployit.booter.local;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;
import com.google.common.base.Predicate;
import com.google.common.collect.Iterators;

import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.reflect.DescriptorRegistry;
import com.xebialabs.deployit.plugin.api.udm.ConfigurationItem;

import nl.javadude.scannit.Scannit;

import static com.google.common.base.Predicates.equalTo;
import static com.google.common.collect.FluentIterable.from;
import static com.google.common.collect.Lists.newArrayList;
import static com.xebialabs.deployit.booter.local.SyntheticHelper.readSyntheticDocument;
import static com.xebialabs.deployit.booter.local.utils.XmlUtils.childByName;
import static com.xebialabs.xlplatform.utils.ClassLoaderUtils$.MODULE$;

class TypeSystemBootstrapper {

    void bootstrap(TypeDefinitions typeDefs) {
        logger.info("Bootstrapping the XL Deploy Type System.");
        scanClasses(typeDefs);
        scanSynthetics(typeDefs);
        createTypeTree(typeDefs);
        registerTypeDefinitions(typeDefs);
    }

    private void createTypeTree(TypeDefinitions typeDefs) {
        for (TypeDefinitions.TypeDefinition typeDefinition : typeDefs.getDefinitions()) {
            typeDefinition.registerTypeTree();
        }
    }

    void verifyTypes(Verifications verifications) {
        for (Descriptor descriptor : from(DescriptorRegistry.getDescriptors()).filter(new Predicate<Descriptor>() {
            public boolean apply(Descriptor input) {
                return !input.isVirtual();
            }
        })) {
            ((LocalDescriptor) descriptor).verify(verifications);
        }
    }

    private void scanClasses(TypeDefinitions typeDefs) {
        typeDefs.defineType(ConfigurationItem.class);
        Set<Class<? extends ConfigurationItem>> cis = Scannit.getInstance().getSubTypesOf(ConfigurationItem.class);
        for (Class<? extends ConfigurationItem> ci : cis) {
            typeDefs.defineType(ci);
        }
    }

    private void scanSynthetics(TypeDefinitions typeDefs) {
        try {
            List<Element> types = newArrayList();
            List<Element> typeModifications = newArrayList();
            readSynthetics("synthetic.xml", types, typeModifications);
            readSynthetics("synthetic-test.xml", types, typeModifications);

            parseAllTypeDefinitions(types, typeDefs);
            parseAllTypeModifications(typeDefs, typeModifications);
        } catch (IOException ex) {
            throw new RuntimeException("Could not read synthetic type definitions.", ex);
        }
    }

    private void registerTypeDefinitions(TypeDefinitions typeDefs) {
        for (TypeDefinitions.TypeDefinition typeDefinition : typeDefs.getDefinitions()) {
            typeDefinition.register(typeDefs);
        }
    }

    private void parseAllTypeDefinitions(List<Element> types, TypeDefinitions typeDefs) {
        for (Element type : types) {
            typeDefs.defineType(type);
            Iterator<Element> generatedTypes = childByName(type, equalTo("generate-deployable"));
            if (generatedTypes.hasNext()) {
                typeDefs.defineGeneratedType(generatedTypes.next(), type);
            }

            findAllGeneratedParameterTypes(typeDefs, type);
        }
    }

    private void findAllGeneratedParameterTypes(final TypeDefinitions typeDefs, final Element type) {
        Iterator<Element> methodDefs = childByName(type, equalTo("method"));
        while (methodDefs.hasNext()) {
            Element methodDef = methodDefs.next();
            Iterator<Element> parameters = childByName(methodDef, equalTo("parameters"));
            if (parameters.hasNext()) {
                typeDefs.defineGeneratedParameterType(methodDef, type);
            }
        }
    }

    private void parseAllTypeModifications(TypeDefinitions typeDefs, List<Element> typeModifications) {
        for (Element typeModification : typeModifications) {
            typeDefs.modifyType(typeModification);
            findAllGeneratedParameterTypes(typeDefs, typeModification);
        }
    }

    private void readSynthetics(String name, List<Element> typeDefinitionCollector, List<Element> typeModificationsCollector) throws IOException {
        Enumeration<URL> syntheticXMLs = MODULE$.classLoader().getResources(name);
        while (syntheticXMLs.hasMoreElements()) {
            final URL syntheticXML = syntheticXMLs.nextElement();
            logger.debug("Scanning synthetic XML: {}", syntheticXML);
            Element docElement = readSyntheticDocument(syntheticXML).getDocumentElement();

            Iterators.addAll(typeDefinitionCollector, childByName(docElement, equalTo("type")));
            Iterators.addAll(typeModificationsCollector, childByName(docElement, equalTo("type-modification")));
        }
    }

    private static final Logger logger = LoggerFactory.getLogger(TypeSystemBootstrapper.class);
}
